from zeep.wsse.username import UsernameToken
from zeep import Client


# region Returns workday client
# Details are provided in Credentials.py file
def get_client(tenant_host, tenant_id, webservice, webservice_version, username, password):
    """
    :param tenant_host:  Tenant Host Name
    :param tenant_id: Tenant ID
    :param webservice: Webservice name like, Human_Resources, Staffing, Resource_Management, Financial_Management etc
    :param webservice_version: Webservice Version, Currently tested with v35.0
    :param username: Integration username
    :param password: Integration user password
    :return: Zeep client object
    """
    if "https://" not in tenant_host:
        tenant_host = "https://" + tenant_host
    if "@" + tenant_id not in username:
        username = username + "@" + tenant_id
    # Build client url
    client_url = tenant_host + "/ccx/service/" + tenant_id + "/" + webservice + "/" + webservice_version + "?wsdl"
    return Client(client_url, UsernameToken(username, password))


def get_raas_client(tenant_host, tenant_id, raas, raas_user, username, password):
    """
    :param tenant_host:  Tenant Host Name
    :param tenant_id: Tenant ID
    :param raas:Report as service Reference ID
    :param raas_user: The username whos has this report in ownership
    :param username: Integration username
    :param password: Integration user password
    :return: Zeep client object
    """
    if "https://" not in tenant_host:
        tenant_host = "https://" + tenant_host
    if "@" + tenant_id not in username:
        username = username + "@" + tenant_id
    # Build client url
    client_url = tenant_host + "/ccx/service/Report2/" + tenant_id + "/" + raas_user + "/" + raas + "?wsdl"

    return Client(client_url, UsernameToken(username, password))



import xmltodict
from collections import OrderedDict
from io import StringIO
import json
from lxml import etree


def parse_xml_response(response):
    rsp_txt=response.text.replace("<?xml version='1.0' encoding='UTF-8'?>", '')
    rsp_txt=rsp_txt.replace('<?xml version="1.0" encoding="utf-8"?>', '')
    return etree.parse(StringIO(rsp_txt))


def transformedresponse(result, transform):
    doc = parse_xml_response(result)
    result_tree = transform(doc)
    return xmltodict.parse(result_tree)


def print_to_console_call_details(response):
    print("current_page: " + response['root']['Page'])
    print("total_pages: " + response['root']['Total_Pages'])
    print("total_results: " + response['root']['Total_Results'])


def prepare_response(transformed_response, final_response_result):
    if transformed_response['root']['records'] is not None:
        if isinstance(transformed_response['root']['records']['record'], OrderedDict):
            final_response_result = final_response_result + json.loads(
                json.dumps([transformed_response['root']['records']['record']]))
        else:
            final_response_result = final_response_result + json.loads(
                json.dumps(transformed_response['root']['records']['record']))
    return final_response_result




import datetime
import sys
import pytz
import zeep
from zeep import xsd
from wwsclient.util import *
import zeep.exceptions

"""
__author__      = Hikmat Ullah
__copyright__   = Copyright 2021
__date__        = 18, January 2021
"""

header = xsd.Element(
    '{urn:com.workday/bsvc}Workday_Common_Header',
    xsd.ComplexType([
        xsd.Element(
            '{urn:com.workday/bsvc}Include_Reference_Descriptors_In_Response',
            xsd.Boolean()),
    ])
)
workday_common_header = header(Include_Reference_Descriptors_In_Response=True)
current_date = datetime.datetime.now(pytz.timezone('US/Pacific')).strftime("%Y-%m-%dT%H:%M:%S")


# region Get Operation
def get_method(client, request, xslt_code, operation, print_to_console=False, count=100, add_response_filter=True):
    """
    :param client: Zeep client
    :param request: request object
    :param xslt_code: xslt code as string
    :param operation: Webservice Operation
    :param print_to_console: print page numbers fetched in console
    :param count: count returned per page
    :param add_response_filter: control to add response filter in request or not. this is necessary when report is called
    :return: All suppliers returned from api
    """
    transform = etree.XSLT(etree.XML(xslt_code))
    total_pages = 1
    current_page = 0
    final_response_result = []
    if print_to_console:
        print("calling workday for operation: " + operation + ", with results per page: " + str(count))

    while current_page < total_pages:
        try:
            request['_soapheaders'] = [workday_common_header]
            if add_response_filter:
                request['Response_Filter'] = {
                    "As_Of_Effective_Date": current_date,
                    "As_Of_Entry_DateTime": current_date,
                    "Page": current_page + 1,
                    "Count": count
                }

            with client.settings(raw_response=True):
                result = client.service[operation](**request)

            if result.status_code == 200:
                transformed_response = transformedresponse(result, transform)
                if transformed_response['root']['Total_Results'] == '0':
                    print("No data returned by the API!")
                    return

                total_pages = int(transformed_response['root']['Total_Pages'])
                current_page = int(transformed_response['root']['Page'])

                if print_to_console:
                    print_to_console_call_details(transformed_response)
                final_response_result = prepare_response(transformed_response, final_response_result)
            elif result.status_code == 500:
                print(result.text)
                return

        except zeep.exceptions.Fault as ex:
            print("error in " + operation + " : " + str(ex))
            print("Unexpected error: ", sys.exc_info()[0])
            break

        except zeep.exceptions.XMLParseError as ex:
            print("xml error in " + operation + " : " + str(ex))
            print("Unexpected error: ", sys.exc_info()[0])
            current_page = current_page + 1

        except Exception as ex:
            print("generic error in " + operation + " : " + str(ex))
            print("Unexpected error: ", sys.exc_info()[0])
            break

    return final_response_result


# endregion

# region Edit, Delete, Submit, Import, ADD, PUT methods
def crud_method(client, request, operation):
    request['_soapheaders'] = [workday_common_header]
    return client.service[operation](**request)
# endregion



import os.path
from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name='wwsclient',
    version='0.0.0.8',
    description='This script will be expose all workday webservice operations',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/hikmat30ce/Workday-Webservices-Client.git',
    author='Hikmat Ullah',
    author_email='me@hikmatu.com',
    license='MIT',
    packages=['wwsclient'],
    include_package_data=True,
    install_requires=[
        'appdirs==1.4.4',
        'attrs==20.3.0',
        'cached-property==1.5.2',
        'certifi==2020.12.5',
        'chardet==4.0.0',
        'defusedxml==0.7.0',
        'idna==2.10',
        'isodate==0.6.0',
        'lxml==4.6.2',
        'pytz==2021.1',
        'requests==2.25.1',
        'requests-file==1.5.1',
        'requests-toolbelt==0.9.1',
        'six==1.15.0',
        'urllib3==1.26.3',
        'xmltodict==0.12.0',
        'zeep==4.0.0'],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.9',
    ],
    entry_points={"console_scripts": ["realpython=reader.__main__:main"]},

    # install_requires={
    #         "dist": [
    #             'appdirs==1.4.4',
    #             'attrs==20.3.0',
    #             'cached-property==1.5.2',
    #             'certifi==2020.12.5',
    #             'chardet==4.0.0',
    #             'defusedxml==0.7.0',
    #             'idna==2.10',
    #             'isodate==0.6.0',
    #             'lxml==4.6.2',
    #             'pytz==2021.1',
    #             'requests==2.25.1',
    #             'requests-file==1.5.1',
    #             'requests-toolbelt==0.9.1',
    #             'six==1.15.0',
    #             'urllib3==1.26.3',
    #             'xmltodict==0.12.0',
    #             'zeep==4.0.0'
    #         ],
    #         "dev": [
    #             'appdirs==1.4.4',
    #             'attrs==20.3.0',
    #             'cached-property==1.5.2',
    #             'certifi==2020.12.5',
    #             'chardet==4.0.0',
    #             'defusedxml==0.7.0',
    #             'idna==2.10',
    #             'isodate==0.6.0',
    #             'lxml==4.6.2',
    #             'pytz==2021.1',
    #             'requests==2.25.1',
    #             'requests-file==1.5.1',
    #             'requests-toolbelt==0.9.1',
    #             'six==1.15.0',
    #             'urllib3==1.26.3',
    #             'xmltodict==0.12.0',
    #             'zeep==4.0.0'
    #         ]
    #     },
)
